/**
 * Copyright (C) 2010 Hamburg Sud
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.hamburgsud.aludratest.action.interaction;

import java.awt.event.KeyEvent;

import org.w3c.dom.NodeList;

import com.hamburgsud.aludratest.action.verification.Verification;
import com.hamburgsud.aludratest.caller.CommandCallerRef;
import com.hamburgsud.aludratest.handler.ExceptionHandlerRef;
import com.hamburgsud.aludratest.locator.LocatorProvider;
import com.hamburgsud.aludratest.locator.element.XPathLocator;
import com.hamburgsud.aludratest.locator.option.OptionLocator;
import com.hamburgsud.log4testing.core.data.TestStatus;
import com.hamburgsud.log4testing.core.data.TestStep;

/**
 * The interface {@link Interaction} provides several methods to interact with
 * the active screen of the application under test. This means that the
 * application under test can be controlled with the help of these methods.
 * Every class which implements this interface must assure that a call of one of
 * these methods results in a interaction with the application under test.<br/>
 * Methods with locators accept two types of locators.
 * <ol>
 * <li>The first type is a by the caller typed locator.</li>
 * <li>The second type is just a String which will be automatically transformed
 * into the default locator. In which locator the String will be transformed
 * depends on the implementation of each method of this interface. Normally
 * these methods call
 * {@link LocatorProvider#getDefaultElementLocator(net.sourceforge.aludratest.wrapper.Wrapper, String)}
 * to get the default locator for the corresponding wrapper.</li>
 * </ol>
 * This mechanism exists also for methods with other types of locators. One
 * example is the {@link OptionLocator} of the method
 * {@link #selectDropDownEntry(String, String)} where the second parameter will
 * automatically be transformed into the default locator of this operation and
 * the underlying wrapper implementation.<br/>
 * For verifications of the application under test see {@link Verification}.
 */
public interface Interaction extends ExceptionHandlerRef, CommandCallerRef {

	/**
	 * Selects a radiobutton. If this radiobutton belongs to a group of
	 * radiobuttons, the other radiobuttons of this group will be deselected.
	 * 
	 * @param locator
	 *            to locate one specific radiobutton in the application under
	 *            test
	 * @return a logging object which contains information about the execution
	 *         of this action. This object is normally just for the purpose of
	 *         logging
	 */
	public TestStep selectRadiobutton(final Object locator);

	/**
	 * Changes the selection state of a checkbox. If this method will be called
	 * on a checkbox which is still selected, the checkbox will be deselected.
	 * 
	 * @param locator
	 *            to locate one specific checkbox in the application under test
	 * @return a logging object which contains information about the execution
	 *         of this action. This object is normally just for the purpose of
	 *         logging
	 */
	public TestStep changeCheckbox(final Object locator);

	/**
	 * Selects a checkbox. If the checkbox is already selected, this method will
	 * do nothing.
	 * 
	 * @param locator
	 *            to locate one specific checkbox in the application under test
	 * @return a logging object which contains information about the execution
	 *         of this action. This object is normally just for the purpose of
	 *         logging
	 */
	public TestStep selectCheckbox(final Object locator);

	/**
	 * Deselects a checkbox. If the checkbox is not selected, this method will
	 * do nothing.
	 * 
	 * @param locator
	 *            to locate one specific checkbox in the application under test
	 * @return a logging object which contains information about the execution
	 *         of this action. This object is normally just for the purpose of
	 *         logging
	 */
	public TestStep deselectCheckbox(final Object locator);

	/**
	 * Selects an entry in a dropdownbox with the help of a
	 * <code>OptionLocator</code>. First it locates the element with the help of
	 * the <code>locator</code>, then it tries to select an entry defined by
	 * <code>optionLocator</code>.
	 * 
	 * @param locator
	 *            to locate one specific dropdownbox in the application under
	 *            test
	 * @param optionLocator
	 *            defines which entry of the located dropdownbox shall be
	 *            selected
	 * @return a logging object which contains information about the execution
	 *         of this action. This object is normally just for the purpose of
	 *         logging
	 */
	public TestStep selectDropDownEntry(final Object locator,
			final Object optionLocator);

	/**
	 * Types in some text without conversion/manipulation of the passed through
	 * text. The content of the locator will be deleted, if the expected text is
	 * not set.
	 * 
	 * @param locator
	 *            to locate one specific inputfield in the application under
	 *            test. An inputfield is any GUI element which accepts user
	 *            inputs.
	 * @param text
	 *            which shall be typed in without conversion/manipulation
	 * @return a logging object which contains information about the execution
	 *         of this action. This object is normally just for the purpose of
	 *         logging
	 */
	public TestStep type(final Object locator, final String text);

	/**
	 * Clicks with a single click on any kind of element which reacts on click
	 * events. A common example are buttons and links.
	 * 
	 * @param locator
	 *            to locate buttons, links or any other elements which react on
	 *            mouse clicks.
	 * @return a logging object which contains information about the execution
	 *         of this action. This object is normally just for the purpose of
	 *         logging
	 */
	public TestStep click(final Object locator);

	/**
	 * Reads the value of an inputfield and returns it as a String without
	 * conversion/manipulation.
	 * 
	 * @param locator
	 *            to locate the inputfield in the application under test where
	 *            the inputfield must be an element for user inputs. Two
	 *            examples are single line inputfields and text areas in web
	 *            applications. This action works also with disabled
	 *            inputfields.
	 * @return the value of the inputfield. If the inputfield could not be
	 *         found, <code>null</code> will be returned
	 */
	public String getInputFieldValue(final Object locator);

	/**
	 * Selects a window using a window locator. Once a window has been selected,
	 * all commands go to that window.
	 * 
	 * @param locator
	 *            to locate one specific window of the application under test
	 * @return a logging object which contains information about the execution
	 *         of this action. This object is normally just for the purpose of
	 *         logging
	 */
	public TestStep selectWindow(final Object locator);

	/**
	 * Gets the text of an element.
	 * 
	 * @param locator points to one element
	 * @return the unmodified text of an element
	 */
	public String getText(final Object locator);

	/**
	 * Closes all opened windows.
	 * 
	 * @return a logging object which contains information about the execution
	 *         of this action. This object is normally just for the purpose of
	 *         logging
	 */
	public TestStep tearDown();

	/**
	 * Opens a new browser window with the given url and waits until the page is
	 * fully loaded.
	 * 
	 * @param url
	 * @return a logging object which contains information about the execution
	 *         of this action. This object is normally just for the purpose of
	 *         logging
	 */
	public TestStep open(final String url);

	/**
	 * Refreshes the page of the currently selected window and waits until the
	 * page is fully loaded.
	 * 
	 * @return a logging object which contains information about the execution
	 *         of this action. This object is normally just for the purpose of
	 *         logging
	 */
	public TestStep refresh();

	/**
	 * Maximizes the currently selected window.
	 * 
	 * @return a logging object which contains information about the execution
	 *         of this action. This object is normally just for the purpose of
	 *         logging
	 */
	public TestStep windowMaximize();

	/**
	 * Gives focus to the currently selected window.
	 * 
	 * @return a logging object which contains information about the execution
	 *         of this action. This object is normally just for the purpose of
	 *         logging
	 */
	public TestStep windowFocus();

	/**
	 * Waits until a page is fully loaded. Normally every wrapper defines its
	 * own timeout. When this timeout exceeds, this method returns an object of
	 * the type <code>TestStep</code> which should have one of the status
	 * {@link TestStatus#ERROR} or {@link TestStatus#EXCEPTION}. For more
	 * information about the timeout see the used wrapper.
	 * 
	 * @return a logging object which contains information about the execution
	 *         of this action. This object is normally just for the purpose of
	 *         logging. If the timeout exceeds this object should have one of
	 *         the status {@link TestStatus#ERROR} or
	 *         {@link TestStatus#EXCEPTION}.
	 */
	public TestStep waitForPageToLoad();

	/**
	 * Gives focus on an element.
	 * 
	 * @param locator
	 *            of the element which shall get the focus
	 * @return a logging object which contains information about the execution
	 *         of this action. This object is normally just for the purpose of
	 *         logging
	 */
	public TestStep focus(final Object locator);

	/**
	 * Presses a key on the keyboard.
	 * 
	 * @param keycode
	 *            is the key which shall be pressed. See {@link KeyEvent} for
	 *            available keycodes
	 * @return a logging object which contains some information about the
	 *         execution of this action.
	 */
	public TestStep keyPress(final int keycode);

	/**
	 * Does a double click on the element which is identified by the locator.
	 * 
	 * @param locator
	 *            which identifies the element which shall be double clicked
	 * @return a logging object which contains information about the success of
	 *         the execution of this action.
	 */
	public TestStep doubleClick(final Object locator);

	/**
	 * Closes all open windows which do not have the title.
	 * 
	 * @param locator
	 *            is a window locator or just a String which will be
	 *            automatically converted to one of the default locators
	 *            depending on the underlying driver and the used default
	 *            localization mechanism
	 * @return a logging object which contains information about the success of
	 *         the execution of this action.
	 */
	public TestStep closeOtherWindows(final Object locator);

	/**
	 * Counts the elements found with an XPath expression.
	 * 
	 * @param rowCountXPath
	 *            XPath expression
	 * @return how many element could be find with the passed through XPath
	 *         expression
	 * @deprecated Use {@link #evalXPath(String)} instead with the XPath
	 *             function count(). This method will be removed in one of the
	 *             future releases.
	 */
	@Deprecated
	public int getXPathCount(String rowCountXPath);

	/**
	 * Does the same like {@link #evalXPath(String)} does.
	 * 
	 * @param locator whose XPath expression will be taken and evaluated
	 * @return NodeList object containing XPath evaluation result
	 */
	public NodeList evalXPath(XPathLocator locator);
	
	/**
	 * Evaluates arbitrary XPath and outputs its result as NodeList object.
	 * 
	 * @param locator
	 *            XPath expressions which will be executed
	 * @return NodeList object containing XPath evaluation result
	 */
	public NodeList evalXPath(String xpath);

}
