/**
 * Copyright (C) 2010 Hamburg Sud
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.hamburgsud.aludratest.action.interaction.selenium;

import static com.hamburgsud.aludratest.locator.LocatorProvider.getDefaultElementLocator;
import static com.hamburgsud.aludratest.locator.LocatorProvider.getDefaultOptionLocator;
import static com.hamburgsud.aludratest.locator.LocatorProvider.getDefaultWindowLocator;

import java.io.ByteArrayInputStream;
import java.io.InputStream;

import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.w3c.tidy.Tidy;

import com.hamburgsud.aludratest.action.Action;
import com.hamburgsud.aludratest.action.interaction.Interaction;
import com.hamburgsud.aludratest.caller.Command;
import com.hamburgsud.aludratest.caller.TearDownCommand;
import com.hamburgsud.aludratest.exception.AludraTestException;
import com.hamburgsud.aludratest.exception.functional.verification.IndexNotFoundException;
import com.hamburgsud.aludratest.exception.functional.verification.LabelsContainsNotAsExpectedException;
import com.hamburgsud.aludratest.exception.technical.LocatorNotSupportedException;
import com.hamburgsud.aludratest.exception.technical.TechnicalException;
import com.hamburgsud.aludratest.handler.DefaultInteractionExceptionHandler;
import com.hamburgsud.aludratest.locator.element.ElementLocator;
import com.hamburgsud.aludratest.locator.element.XPathLocator;
import com.hamburgsud.aludratest.locator.option.IndexLocator;
import com.hamburgsud.aludratest.locator.option.OptionLocator;
import com.hamburgsud.aludratest.locator.window.WindowLocator;
import com.hamburgsud.aludratest.wrapper.Wrapper;
import com.hamburgsud.aludratest.wrapper.selenium.Condition;
import com.hamburgsud.log4testing.core.data.TestCase;
import com.hamburgsud.log4testing.core.data.TestStep;

public class SeleniumInteraction extends Action implements Interaction {

	public SeleniumInteraction(final Wrapper wrapper, final TestCase testCase) {
		super(wrapper, testCase);
		setExceptionHandler(new DefaultInteractionExceptionHandler());
	}

	public TestStep changeCheckbox(final Object locator) {
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,
				locator);
		return getCommandCaller().call(new Command() {

			public void call() throws Exception {
				wrapper.click(elementLocator);
			}

			public String getName() {
				return "changeCheckbox";
			}

			@Override
			public String getMessage() {
				return getLocatorMessage(elementLocator);
			}

		});
	}

	public TestStep deselectCheckbox(final Object locator) {
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,
				locator);
		return getCommandCaller().call(new Command() {

			@Override
			public void call() throws Exception {
				if (wrapper.isChecked(elementLocator)) {
					wrapper.click(elementLocator);
				}
			}

			@Override
			public String getName() {
				return "deselectCheckbox";
			}

			@Override
			public String getMessage() {
				return getLocatorMessage(elementLocator);
			}

		});
	}

	public String getInputFieldValue(final Object locator) {
		final StringBuilder inputFieldValue = new StringBuilder(0);
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,
				locator);
		getCommandCaller().call(new Command() {

			@Override
			public void call() throws Exception {
				wrapper.retryUntilTimeout(new Condition() {
					public boolean eval() throws LocatorNotSupportedException {
						String actualValue =null;
						try {
							actualValue = wrapper.getValue(elementLocator);
						} catch (AludraTestException e) {
						}
						if (actualValue == null || "".equals(actualValue)) {
							return false;
						}
						return true;
					}});
				final String readValue = wrapper.getValue(elementLocator);
				if (readValue != null) {
					inputFieldValue.append(readValue);
				}
			}

			@Override
			public String getName() {
				return "getInputFieldValue";
			}

			@Override
			public String getMessage() {
				return getSeperatedString(getArgumentMessage(inputFieldValue
						.toString()), getLocatorMessage(elementLocator));
			}

		});
		return inputFieldValue.toString();
	}
	
	public String getText(final Object locator) {
		final StringBuilder textValue = new StringBuilder(0);
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,
				locator);
		getCommandCaller().call(new Command() {

			@Override
			public void call() throws Exception {
				final String readValue = wrapper.getText(elementLocator);
				if (readValue != null) {
					textValue.append(readValue);
				}
			}

			@Override
			public String getName() {
				return "getText";
			}

			@Override
			public String getMessage() {
				return getSeperatedString(getArgumentMessage(textValue
						.toString()), getLocatorMessage(elementLocator));
			}

		});
		return textValue.toString();
	}

	public TestStep selectCheckbox(final Object locator) {
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,
				locator);
		return getCommandCaller().call(new Command() {

			@Override
			public void call() throws Exception {
				if (!wrapper.isChecked(elementLocator)) {
					wrapper.click(elementLocator);
				}
			}

			@Override
			public String getName() {
				return "selectCheckbox";
			}

			@Override
			public String getMessage() {
				return getLocatorMessage(elementLocator);
			}

		});
	}

	public TestStep selectDropDownEntry(final Object locator,
			final Object optionLocator) {
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,
				locator);
		final OptionLocator optLocator = getDefaultOptionLocator(wrapper,
				optionLocator);
		return getCommandCaller().call(new Command() {

			@Override
			public void call() throws Exception {
				wrapper.isElementPresent(elementLocator);
				checkLabels(optionLocator, elementLocator);
				wrapper.select(elementLocator, optLocator);
			}

			@Override
			public String getName() {
				return "selectDropDownEntry";
			}

			@Override
			public String getMessage() {
				return getLocatorMessage(elementLocator, optLocator);
			}

		});
	}

	public TestStep selectRadiobutton(final Object locator) {
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,
				locator);
		return getCommandCaller().call(new Command() {

			@Override
			public void call() throws Exception {
				wrapper.click(elementLocator);
			}

			@Override
			public String getName() {
				return "selectRadiobutton";
			}

			@Override
			public String getMessage() {
				return getLocatorMessage(elementLocator);
			}

		});
	}

	public TestStep type(final Object locator, final String text) {
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,
				locator);
		return getCommandCaller().call(new Command() {

			@Override
			public void call() throws Exception {
				wrapper.type(elementLocator, text==null? "" : text);
			}

			@Override
			public String getName() {
				return "type";
			}

			@Override
			public String getMessage() {
				return getSeperatedString(getArgumentMessage(text),
						getLocatorMessage(elementLocator));
			}

		});
	}

	public TestStep open(final String url) {
		return getCommandCaller().call(new Command() {

			@Override
			public void call() throws Exception {
				wrapper.open(url);
			}

			@Override
			public String getName() {
				return "open";
			}

			@Override
			public String getMessage() {
				return url;
			}

		});
	}

	public TestStep refresh() {
		return getCommandCaller().call(new Command() {

			@Override
			public void call() throws Exception {
				wrapper.refresh();
			}

			@Override
			public String getName() {
				return "refresh";
			}

			@Override
			public String getMessage() {
				return "";
			}

		});
	}

	public TestStep tearDown() {
		return getCommandCaller().call(new TearDownCommand() {

			@Override
			public void call() throws Exception {
				wrapper.tearDown();
			}

			@Override
			public String getName() {
				return "tearDown";
			}

			@Override
			public String getMessage() {
				return "";
			}

		});
	}

	public TestStep waitForPageToLoad() {
		return getCommandCaller().call(new Command() {

			@Override
			public void call() throws Exception {
				wrapper.waitForPageToLoad();
			}

			@Override
			public String getName() {
				return "waitForPageToLoad";
			}

			@Override
			public String getMessage() {
				return "";
			}

		});
	}

	public TestStep windowFocus() {
		return getCommandCaller().call(new Command() {

			@Override
			public void call() throws Exception {
				wrapper.windowFocus();
			}

			@Override
			public String getName() {
				return "windowFocus";
			}

			@Override
			public String getMessage() {
				return "";
			}

		});
	}

	public TestStep windowMaximize() {
		return getCommandCaller().call(new Command() {

			@Override
			public void call() throws Exception {
				wrapper.windowMaximize();
			}

			@Override
			public String getName() {
				return "windowMaximize";
			}

			@Override
			public String getMessage() {
				return "";
			}

		});
	}

	public TestStep selectWindow(final Object locator) {
		final WindowLocator windowLocator = getDefaultWindowLocator(wrapper,
				locator);
		return getCommandCaller().call(new Command() {

			@Override
			public void call() throws Exception {
				wrapper.selectWindow(windowLocator);
			}

			@Override
			public String getName() {
				return "selectWindow";
			}

			@Override
			public String getMessage() {
				return getLocatorMessage(windowLocator);
			}

		});
	}

	public TestStep focus(final Object locator) {
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,
				locator);
		return getCommandCaller().call(new Command() {

			@Override
			public void call() throws Exception {
				wrapper.focus(elementLocator);
			}

			@Override
			public String getMessage() {
				return elementLocator.toString();
			}

			@Override
			public String getName() {
				return "focus";
			}
		});
	}

	public TestStep keyPress(final int keycode) {
		return getCommandCaller().call(new Command() {

			@Override
			public String getName() {
				return "keyPress";
			}

			@Override
			public String getMessage() {
				return String.valueOf(keycode);
			}

			@Override
			public void call() throws Exception {
				wrapper.keyPress(keycode);
			}
		});
	}

	public TestStep doubleClick(final Object locator) {
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,
				locator);
		return getCommandCaller().call(new Command() {

			@Override
			public void call() throws Exception {
				wrapper.doubleClick(elementLocator);
			}

			@Override
			public String getName() {
				return "doubleClick";
			}

			@Override
			public String getMessage() {
				return getLocatorMessage(elementLocator);
			}

		});
	}

	public TestStep click(final Object locator) {
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,
				locator);
		return getCommandCaller().call(new Command() {

			@Override
			public void call() throws Exception {
				wrapper.click(elementLocator);
			}

			@Override
			public String getName() {
				return "click";
			}

			@Override
			public String getMessage() {
				return getLocatorMessage(elementLocator);
			}

		});
	}

	public TestStep closeOtherWindows(final Object locator) {
		final WindowLocator stillOpenedWindow = getDefaultWindowLocator(
				wrapper, locator);
		return getCommandCaller().call(new Command() {

			@Override
			public void call() throws Exception {
				final String[] windowTitles = wrapper.getAllWindowTitles();
				for (String windowTitle : windowTitles) {
					final WindowLocator windowToClose = getDefaultWindowLocator(
							wrapper, windowTitle);
					if (!windowToClose.equals(stillOpenedWindow)) {
						wrapper.selectWindow(windowToClose);
						wrapper.close();
					}
				}
				wrapper.selectWindow(stillOpenedWindow);
			}

			@Override
			public String getName() {
				return "closeOtherWindows";
			}

			@Override
			public String getMessage() {
				return getLocatorMessage(stillOpenedWindow);
			}
		});

	}

	@Deprecated
	public int getXPathCount(String xpath) {
		return wrapper.getXPathCount(xpath).intValue();
	}

	private void checkLabels(final Object optLocator,
			final ElementLocator elementLocator) throws TechnicalException,
			LocatorNotSupportedException, AludraTestException,
			LabelsContainsNotAsExpectedException, IndexNotFoundException {
		final boolean isStringOption = optLocator instanceof String;
		wrapper.retryUntilTimeout(new Condition() {
			public boolean eval() throws LocatorNotSupportedException {
				String[] actualLabels = wrapper.getLabels(elementLocator);
				if (isStringOption) {
					final String missmatches;
					missmatches = containsString((String) optLocator,
							actualLabels);
					if (missmatches.length() > 0) {
						return false;
					}
					return true;
				} else {
					if (optLocator instanceof IndexLocator)
						if (actualLabels.length > ((IndexLocator) optLocator)
								.getIndex())
							return true;
					return false;
				}
			}
		});
		final String[] actualLabels = wrapper.getLabels(elementLocator);
		if (isStringOption) {
			final String missmatches;
			missmatches = containsString((String) optLocator, actualLabels);
			if (missmatches.length() > 0) {
				throw new LabelsContainsNotAsExpectedException(elementLocator,
						missmatches.toString());
			}
		} else {
			if (optLocator instanceof IndexLocator)
				if (!(actualLabels.length >= ((IndexLocator) optLocator)
						.getIndex()))
					throw new IndexNotFoundException(elementLocator,
							((IndexLocator) optLocator).getIndex().toString());
		}
	}

	private String containsString(String expectedString, String[] actualStrings) {
		final StringBuilder missmatches = new StringBuilder();
		final StringBuilder actualString = new StringBuilder();
		String combinedActualString = null;
		for (int i = 0; i < actualStrings.length; i++) {
			actualString.append(actualStrings[i]+",");
		}
		combinedActualString = actualString.toString();
		if (!(combinedActualString.contains(expectedString)))
			missmatches.append(expectedString);
		return missmatches.toString();
	}

	public NodeList evalXPath(XPathLocator locator) {
		return evalXPath(locator.toString());
	}
	
	public NodeList evalXPath(final String xpath) {
		final Object[] result = new Object[1];
		getCommandCaller().call(new Command() {
			
			@Override
			public void call() throws Exception {
				String htmlSource = new String(wrapper.getPageSource()
						.getFileData());
				Tidy tidy = new Tidy();
				tidy.setQuiet(true);
				tidy.setShowWarnings(false);
				tidy.setShowErrors(0);
				tidy.setXmlOut(false);
				InputStream inputStream = new ByteArrayInputStream(htmlSource
						.getBytes());
				Document document = tidy.parseDOM(inputStream, null);
				inputStream.close();
				XPathExpression xPathExpression = XPathFactory.newInstance()
						.newXPath().compile(xpath);
				result[0] = (NodeList) xPathExpression.evaluate(document,
						XPathConstants.NODESET);
			}
			
			@Override
			public String getName() {
				return "evaluateXPath";
			}
			
			@Override
			public String getMessage() {
				return xpath;
			}
			
		});
		return (NodeList) result[0];
	}

}
