/**
 * Copyright (C) 2010 Hamburg Sud
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.hamburgsud.aludratest.action.verification.selenium;

import static com.hamburgsud.aludratest.locator.LocatorProvider.getDefaultElementLocator;
import static com.hamburgsud.aludratest.locator.LocatorProvider.getDefaultElementLocators;

import java.text.NumberFormat;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Locale;
import java.util.Set;

import com.hamburgsud.aludratest.action.Action;
import com.hamburgsud.aludratest.action.verification.Verification;
import com.hamburgsud.aludratest.caller.Command;
import com.hamburgsud.aludratest.configuration.SeleniumWrapperConfiguration;
import com.hamburgsud.aludratest.exception.AludraTestException;
import com.hamburgsud.aludratest.exception.functional.verification.AttributesValueNotAsExpectedException;
import com.hamburgsud.aludratest.exception.functional.verification.CheckBoxOrRadioButtonUncheckedException;
import com.hamburgsud.aludratest.exception.functional.verification.EditableException;
import com.hamburgsud.aludratest.exception.functional.verification.LabelNotAsExpectedException;
import com.hamburgsud.aludratest.exception.functional.verification.LabelsContainsNotAsExpectedException;
import com.hamburgsud.aludratest.exception.functional.verification.LabelsNotAsExpectedException;
import com.hamburgsud.aludratest.exception.functional.verification.NoFocusException;
import com.hamburgsud.aludratest.exception.functional.verification.NotEditableException;
import com.hamburgsud.aludratest.exception.functional.verification.NotVisibleException;
import com.hamburgsud.aludratest.exception.functional.verification.TextNotAsExpected;
import com.hamburgsud.aludratest.exception.functional.verification.ValuesNotAsExpected;
import com.hamburgsud.aludratest.exception.technical.LocatorNotSupportedException;
import com.hamburgsud.aludratest.exception.technical.TechnicalException;
import com.hamburgsud.aludratest.gui.component.Table;
import com.hamburgsud.aludratest.gui.component.Table.Row;
import com.hamburgsud.aludratest.handler.DefaultVerificationExceptionHandler;
import com.hamburgsud.aludratest.locator.Locator;
import com.hamburgsud.aludratest.locator.element.ElementLocator;
import com.hamburgsud.aludratest.wrapper.Wrapper;
import com.hamburgsud.aludratest.wrapper.selenium.Condition;
import com.hamburgsud.log4testing.core.data.TestCase;
import com.hamburgsud.log4testing.core.data.TestStep;

public class SeleniumVerification extends Action implements Verification {

	public SeleniumVerification(final Wrapper wrapper, final TestCase testCase) {
		super(wrapper, testCase);
		setExceptionHandler(new DefaultVerificationExceptionHandler());
	}

	public TestStep isChecked(final Object... locators) {
		final ElementLocator[] elementLocators = getDefaultElementLocators(
				wrapper, locators);
		return getCommandCaller().call(new Command() {

			@Override
			public void call() throws Exception {
				final ArrayList<Locator> notCheckedElements = new ArrayList<Locator>();
				for (final ElementLocator locator : elementLocators) {
					if (!wrapper.isChecked(locator)) {
						notCheckedElements.add(locator);
					}
				}
				if (notCheckedElements.size() > 0) {
					throw new CheckBoxOrRadioButtonUncheckedException(
							notCheckedElements);
				}
			}

			@Override
			public String getName() {
				return "checkBoxOrRadioButtonIsChecked";
			}

			@Override
			public String getMessage() {
				return getLocatorMessage(elementLocators);
			}

		});
	}

	public TestStep dropDownEntrySelection(final Object locator,
			final String expectedLabel) {
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,
				locator);
		return getCommandCaller().call(new Command() {

			@Override
			public void call() throws Exception {
				final String label = wrapper.getSelectedLabel(elementLocator);
				final boolean labelAsExpected = expectedLabel.equals(label);
				if (!labelAsExpected) {
					throw new LabelNotAsExpectedException(elementLocator,
							expectedLabel, label);
				}
			}

			@Override
			public String getName() {
				return "dropDownEntrySelection";
			}

			@Override
			public String getMessage() {
				return getSeperatedString(getArgumentMessage(expectedLabel),
						getLocatorMessage(elementLocator));
			}

		});
	}

	public TestStep isEditable(final Object... locators) {
		final ElementLocator[] elementLocators = getDefaultElementLocators(
				wrapper, locators);
		return getCommandCaller().call(new Command() {

			@Override
			public void call() throws Exception {
				final ArrayList<Locator> notEditableElements = new ArrayList<Locator>();
				for (final ElementLocator locator : elementLocators) {
					final boolean editable = wrapper.isEditable(locator);
					if (!editable) {
						notEditableElements.add(locator);
					}
				}
				if (notEditableElements.size() > 0) {
					throw new NotEditableException((Locator[]) notEditableElements.toArray());
				}
			}

			@Override
			public String getName() {
				return "isEditable";
			}

			@Override
			public String getMessage() {
				return getLocatorMessage(elementLocators);
			}

		});
	}

	public TestStep isNotEditable(final Object... locators) {
		final ElementLocator[] elementLocators = getDefaultElementLocators(
				wrapper, locators);
		return getCommandCaller().call(new Command() {

			@Override
			public void call() throws Exception {
				final ArrayList<Locator> editableElements = new ArrayList<Locator>();
				for (final ElementLocator locator : elementLocators) {
					final boolean editable = wrapper.isEditable(locator);
					if (!editable) {
						editableElements.add(locator);
					}
				}
				if (editableElements.size() > 0) {
					throw new EditableException(editableElements);
				}
			}

			@Override
			public String getName() {
				return "isNotEditable";
			}

			@Override
			public String getMessage() {
				return getLocatorMessage(elementLocators);
			}

		});
	}

	public TestStep isElementPresent(final Object locator) {
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,
				locator);
		return getCommandCaller().call(new Command() {

			@Override
			public void call() throws Exception {
				wrapper.isElementPresent(elementLocator);
			}

			@Override
			public String getName() {
				return "isElementPresent";
			}

			@Override
			public String getMessage() {
				return getLocatorMessage(elementLocator);
			}

		});
	}

	public TestStep text(final Object locator, final String expectedText) {
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,
				locator);
		return getCommandCaller().call(new Command() {

			@Override
			public void call() throws Exception {
				String actualText = wrapper.getText(elementLocator);
				if (actualText == null || actualText.length() == 0) {
					actualText = "";
				}
				String trimmedExpectedText = expectedText;
				if (trimmedExpectedText == null) {
					trimmedExpectedText = "";
				}
				actualText = actualText.trim();
				trimmedExpectedText = trimmedExpectedText.trim();
				if (!actualText.equalsIgnoreCase(trimmedExpectedText)) {
					throw new TextNotAsExpected(elementLocator, actualText,
							trimmedExpectedText);
				}
			}

			@Override
			public String getName() {
				return "text";
			}

			@Override
			public String getMessage() {
				return getSeperatedString(getArgumentMessage(expectedText),
						getLocatorMessage(elementLocator));
			}

		});
	}

	public TestStep visibility(final Object... locators) {
		final ElementLocator[] elementLocators = getDefaultElementLocators(
				wrapper, locators);
		return getCommandCaller().call(new Command() {

			@Override
			public void call() throws Exception {
				final ArrayList<Locator> notVisibleElements = new ArrayList<Locator>();
				for (final ElementLocator locator : elementLocators) {
					final boolean notVisible = wrapper.isElementPresent(locator);
					if (notVisible) {
						notVisibleElements.add(locator);
					}
				}
				if (notVisibleElements.size() > 0) {
					throw new NotVisibleException((Locator[]) notVisibleElements.toArray());
				}
			}

			@Override
			public String getName() {
				return "visibility";
			}

			@Override
			public String getMessage() {
				return getLocatorMessage(elementLocators);
			}

		});
	}

	public TestStep hasFocus(final Object locator) {
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,
				locator);
		return getCommandCaller().call(new Command() {

			@Override
			public String getName() {
				return "hasFocus";
			}

			@Override
			public String getMessage() {
				return getLocatorMessage(elementLocator);
			}

			@Override
			public void call() throws Exception {
				final boolean hasFocus = wrapper.hasFocus(elementLocator);
				if (!hasFocus) {
					throw new NoFocusException(elementLocator);
				}
			}
		});
	}

	public TestStep hasLabels(final Object locator,
			final String[] expectedLabels) {
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,
				locator);
		return getCommandCaller().call(new Command() {

			@Override
			public String getName() {
				return "hasLabels";
			}

			@Override
			public String getMessage() {
				return getSeperatedString(getArgumentMessage(expectedLabels),
						getLocatorMessage(elementLocator));
			}

			@Override
			public void call() throws Exception {
				checkLables(expectedLabels, elementLocator,false);
			}
		});
	}

	private String compareStrings(final String[] expectedStrings,
			final String[] actualStrings) {
		final StringBuilder missmatches = new StringBuilder();
		for (int i = 0; i < actualStrings.length; i++) {
			if (!actualStrings[i].equals(expectedStrings[i])) {
				missmatches.append(expectedStrings[i]).append("!=").append(
						actualStrings[i]).append("; ");
			}
		}
		return missmatches.toString();
	}

	public TestStep hasValues(final Object locator,
			final String[] expectedValues) {
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,
				locator);
		return getCommandCaller().call(new Command() {

			@Override
			public String getName() {
				return "hasValues";
			}

			@Override
			public String getMessage() {
				return getSeperatedString(getArgumentMessage(expectedValues),
						getLocatorMessage(elementLocator));
			}

			@Override
			public void call() throws Exception {
				final String[] actualValues = wrapper.getValues(elementLocator);
				final String missmatches = compareStrings(expectedValues,
						actualValues);
				if (missmatches.length() > 0) {
					throw new ValuesNotAsExpected(elementLocator, missmatches
							.toString());
				}
			}
		});
	}

	public TestStep hasAttributeValue(final Object locator,
			final String attributeName, final String expectedValue) {
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,
				locator);
		return getCommandCaller().call(new Command() {

			@Override
			public void call() throws Exception {
				final String actualValue = wrapper.getAttributeValue(
						elementLocator, attributeName);
				if (actualValue != null && !actualValue.equals(expectedValue)) {
					throw new AttributesValueNotAsExpectedException(
							elementLocator, attributeName, expectedValue,
							actualValue);
				}
			}

			@Override
			public String getMessage() {
				return getSeperatedString(getArgumentMessage(expectedValue),
						getLocatorMessage(elementLocator));
			}

			@Override
			public String getName() {
				return "hasAttributeValue";
			}
		});
	}

	public TestStep getXPathCount(final String xpath, final int expected) {
		return getCommandCaller().call(new Command() {

			@Override
			public void call() throws Exception {
				final int actualValue = wrapper.getXPathCount(xpath).intValue();
				if (actualValue != expected) {
					throw new ValuesNotAsExpected(new ElementLocator(xpath),
							expected + "!=" + actualValue);
				}
			}

			@Override
			public String getMessage() {
				return getSeperatedString(getArgumentMessage(String
						.valueOf(expected)), "Executed XPath expression: "
						+ xpath);
			}

			@Override
			public String getName() {
				return "getXPathCount";
			}
		});
	}

	/**
	 * @deprecated This method will be removed and not replaced
	 */
	@Deprecated
	public TestStep getTable(final Object locator, final int row, final int col, final String expectedText) {	
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,
				locator);
		return getCommandCaller().call(new Command() {

			@Override
			public void call() throws Exception {
				String actualText = wrapper.getTable(elementLocator, row, col);
				if (actualText == null || actualText.length() == 0) {
					actualText = "";
				}
				actualText = actualText.trim();
				
				String trimmedExpectedText = expectedText;
				if (trimmedExpectedText == null) {
					trimmedExpectedText = "";
				}

				if (!actualText.equals(trimmedExpectedText)) {
					throw new TextNotAsExpected(elementLocator, actualText,
							trimmedExpectedText);
				}

			}

			@Override
			public String getName() {
				return "getTable";
			}

			@Override
			public String getMessage() {
				return getSeperatedString(getArgumentMessage(expectedText),
						getLocatorMessage(elementLocator));
			}

		});
	}

	public TestStep isChecked(final boolean expected, final Object locator) {
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,
				locator);
		return getCommandCaller().call(new Command() {

			@Override
			public void call() throws Exception {
				final ArrayList<Locator> notCheckedElements = new ArrayList<Locator>();
				if (wrapper.isChecked(elementLocator) != expected) {
					notCheckedElements.add(elementLocator);
				}
				if (notCheckedElements.size() > 0) {
					throw new CheckBoxOrRadioButtonUncheckedException(
							notCheckedElements);
				}
			}

			@Override
			public String getName() {
				return "checkBoxOrRadioButtonCheckedStatus";
			}

			@Override
			public String getMessage() {
				return getLocatorMessage(elementLocator);
			}

		});
	}

	public TestStep value(final Object locator, final String expectedText) {
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,
				locator);
		return getCommandCaller().call(new Command() {

			@Override
			public void call() throws Exception {
				String actualText = wrapper.getValue(elementLocator);
				if (actualText == null || actualText.length() == 0) {
					actualText = "";
				}
				String trimmedExpectedText = expectedText;
				if (trimmedExpectedText == null) {
					trimmedExpectedText = "";
				}
				actualText = actualText.trim();
				trimmedExpectedText = trimmedExpectedText.trim();
				if (!actualText.equalsIgnoreCase(trimmedExpectedText)) {
					throw new TextNotAsExpected(elementLocator, actualText,
							trimmedExpectedText);
				}
			}

			@Override
			public String getName() {
				return "value";
			}

			@Override
			public String getMessage() {
				return getSeperatedString(getArgumentMessage(expectedText),
						getLocatorMessage(elementLocator));
			}

		});
	}

	/**
	 * @deprecated This method will be removed and not replaced
	 */
	@Deprecated
	public TestStep tableRows(final Object locator, final Set<Row> rows) {
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,
				locator);

		return getCommandCaller().call(new Command() {

			@Override
			public void call() throws Exception {
				if (!rows.isEmpty()) {
					Row firstRow = rows.iterator().next();
					int colCount = firstRow.getColumnCount();
					Set<Row> actualRows = new HashSet<Table.Row>(rows.size());
					
					for (int r = 0; r < rows.size(); r++) {
						Object[] rowData = new Object[colCount];
						for (int c = 0; c < rowData.length; c++) {
							rowData[c] = wrapper.getTable(elementLocator, r+1, c);
							if (firstRow.isNumber(c))
								rowData[c] = parseNumber(rowData[c].toString());
						}
						actualRows.add(new Row(rowData));
					}
				
					if (!rows.containsAll(actualRows)) {
						findFirstDeviation(rows, actualRows);
					}					
				}
			}

			private void findFirstDeviation(Set<Row> rows, Set<Row> actualRows) throws TextNotAsExpected {
				StringBuffer actualRowsBuffer = new StringBuffer();
				for (Row row : actualRows) {
					actualRowsBuffer.append("[").append(row.toString()).append("], ");
				}
				
				StringBuffer rowsBuffer = new StringBuffer();
				for (Row row : rows) {
					rowsBuffer.append("[").append(row.toString()).append("], ");
				}
				
				throw new TextNotAsExpected(elementLocator, actualRowsBuffer.toString(), rowsBuffer.toString());
			}

			@Override
			public String getName() {
				return "tableRows";
			}

			@Override
			public String getMessage() {
				return getLocatorMessage(elementLocator);
			}

		});
	}

	public TestStep text(final Object locator, final Number expectedNumber) {
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,
				locator);
		return getCommandCaller().call(new Command() {

			@Override
			public void call() throws Exception {
				String actualText = wrapper.getText(elementLocator);
				Number actualNumber = parseNumber(actualText);
				Number expNumber = expectedNumber == null ? 0d : expectedNumber;
				if (Math.abs(actualNumber.doubleValue()
						- expNumber.doubleValue()) >= SeleniumWrapperConfiguration.VERIFICATION_NUMERIC_DIFFERENCE) {
					throw new TextNotAsExpected(elementLocator, actualNumber
							.toString(), expNumber.toString());
				}
			}

			@Override
			public String getName() {
				return "text";
			}

			@Override
			public String getMessage() {
				return getSeperatedString(
						getArgumentMessage(String.valueOf(expectedNumber)),
						getLocatorMessage(elementLocator));
			}

		});
	}

	public TestStep value(final Object locator, final Number expectedNumber) {
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,
				locator);
		return getCommandCaller().call(new Command() {

			@Override
			public void call() throws Exception {
				String actualText = wrapper.getValue(elementLocator);
				Number actualNumber = parseNumber(actualText);
				Number expNumber = expectedNumber == null ? 0d : expectedNumber;

				if (Math.abs(actualNumber.doubleValue()
						- expNumber.doubleValue()) >= SeleniumWrapperConfiguration.VERIFICATION_NUMERIC_DIFFERENCE) {
					throw new TextNotAsExpected(elementLocator, actualNumber
							.toString(), expNumber.toString());
				}
			}

			@Override
			public String getName() {
				return "value";
			}

			@Override
			public String getMessage() {
				return getSeperatedString(
						getArgumentMessage(String.valueOf(expectedNumber)),
						getLocatorMessage(elementLocator));
			}

		});
	}

	//TODO To be checked how common this method is
	private Number parseNumber(String actualText) throws ParseException {
		Locale loc = Locale.US;

		if (actualText.matches(".*,[0-9]{1,2}")
				|| actualText.matches(".*\\.[0-9]{3}")) {
			loc = Locale.GERMANY;
		}

		if ("".equals(actualText))
			return 0;

		NumberFormat nf = NumberFormat.getNumberInstance(loc);
		Number n = nf.parse(actualText);
		return n;
	}

	public TestStep containsLabels(Object locator, final String[] labels) {
		final ElementLocator elementLocator = getDefaultElementLocator(wrapper,
				locator);
		return getCommandCaller().call(new Command() {

			@Override
			public String getName() {
				return "containsLabels";
			}

			@Override
			public String getMessage() {
				return getSeperatedString(getArgumentMessage(labels),
						getLocatorMessage(elementLocator));
			}

			@Override
			public void call() throws Exception {
				checkLables(labels, elementLocator,true);
			}

		});
	}

	private String containsStrings(String[] expectedStrings,
			String[] actualStrings) {
		final StringBuilder missmatches = new StringBuilder();
		final StringBuilder actualString = new StringBuilder();
		String combinedActualString = null;
		for (int i = 0; i < actualStrings.length; i++) {
			actualString.append(actualStrings[i]+",");
		}
		combinedActualString = actualString.toString();
		for (int i = 0; i < expectedStrings.length; i++) {
			if (!(combinedActualString.contains(expectedStrings[i])))
				missmatches.append(expectedStrings[i]).append(" ");
		}
		return missmatches.toString();
	}

	private void checkLables(final String[] labels,
			final ElementLocator elementLocator, final boolean contains) throws TechnicalException,
			LocatorNotSupportedException, AludraTestException,
			LabelsContainsNotAsExpectedException {

		wrapper.retryUntilTimeout(new Condition() {
			final String[] actualLabels = wrapper
					.getLabels(elementLocator);
			public boolean eval() throws LocatorNotSupportedException {
				final String missmatches;
				if (contains){
					missmatches = containsStrings(labels,
							actualLabels);
				} else {
					missmatches = compareStrings(labels,
							actualLabels);
				}
		
				if (missmatches.length() > 0) {
					return false;
				}
				return true;
			}
		});
		final String[] actualLabels = wrapper.getLabels(elementLocator);
		final String missmatches = containsStrings(labels, actualLabels);
		if (missmatches.length() > 0) {
			if (contains)
			throw new LabelsContainsNotAsExpectedException(
					elementLocator, missmatches.toString());
			else 
				throw new LabelsNotAsExpectedException(
						elementLocator, missmatches.toString());
		}
	}

}
