/**
 * Copyright (C) 2010 Hamburg Sud
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.hamburgsud.aludratest.caller;

import java.util.HashSet;

import com.hamburgsud.aludratest.handler.ExceptionHandler;
import com.hamburgsud.aludratest.wrapper.Wrapper;
import com.hamburgsud.log4testing.core.data.TestCase;
import com.hamburgsud.log4testing.core.data.TestStatus;
import com.hamburgsud.log4testing.core.data.TestStep;

public class DefaultCommandCaller implements ConditionCommandCaller, CommandCaller {

	private TestCase testCase;
	private ExceptionHandler handler;
	private Wrapper wrapper;
	private static HashSet<TestCase> stoppedExecution;

	static {
		stoppedExecution = new HashSet<TestCase>();
	}

	private DefaultCommandCaller() {
	}

	public DefaultCommandCaller(final TestCase testCase,
			final ExceptionHandler handler, final Wrapper wrapper) {
		this();
		this.testCase = testCase;
		setExceptionHandler(handler);
		this.wrapper = wrapper;
	}

	private boolean continueExecution(final Command command) {
		boolean stopped = false;
		if (stoppedExecution.contains(testCase)) {
			stopped = true;
		}
		if (command instanceof TearDownCommand) {
			stopped = false;
		}
		return !stopped;
	}

	private void stopExecution() {
		stoppedExecution.add(testCase);
	}

	public TestStep call(final Command command) {
		final TestStep testStep = testCase.newTestStep();
		if (continueExecution(command)) {
			execute(command, testStep);
		} else {
			testStep.setStatus(TestStatus.IGNORED);
		}
		testStep.setCommand(command.getName());
		testStep.setMessage(command.getMessage());
		return testStep;
	}
	
	public boolean callCondition(final Command command) {
		if (continueExecution(command))
			return executeCondition(command);
		return false;
	}
	
		
	private boolean executeCondition(final Command command) {
		try {
			command.call();
			return true;
		} catch (Exception e) {
			return false;
		}
	}

	private void execute(final Command command, final TestStep testStep) {
		try {
			command.call();
			testStep.setStatus(TestStatus.OK);
		} catch (Exception e) {
			if (!handler.handle(e, testStep, command, wrapper)) {
				stopExecution();
			}
		}
	}

	public ExceptionHandler getExceptionHandler() {
		return handler;
	}

	public void setExceptionHandler(final ExceptionHandler handler) {
		this.handler = handler;
	}

}
