/**
 * Copyright (C) 2010 Hamburg Sud
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.hamburgsud.aludratest.configuration;

import java.util.List;

import org.apache.commons.configuration.Configuration;
import org.apache.commons.configuration.ConfigurationException;
import org.apache.commons.configuration.PropertiesConfiguration;
import org.apache.log4j.Logger;

@SuppressWarnings("unchecked")
public class SeleniumWrapperConfiguration {

	final private static Logger logger = Logger
			.getLogger(SeleniumWrapperConfiguration.class);

	private static Configuration config = getConfig();

	private static Configuration getConfig() {
		Configuration config = null;
		try {
			config = new PropertiesConfiguration("seleniumWrapper.properties");
		} catch (ConfigurationException e) {
			// When the configuration file cannot be loaded AludraTest is not
			// runnable. That's why we print the stack trace out to the error
			// stream.
			logger.error("The configuration file of the "
					+ "Selenium wrapper could not be loaded.", e);
		}
		return config;
	}

	/**
	 * Hosts where the Selenium rc server runs.
	 */
	final public static List<String> SELENIUM_HOSTS = config
			.getList("selenium.host");

	/**
	 * Port on which the Selenium rc server listens.
	 */
	final public static int SELENIUM_PORT = config.getInt("selenium.port");

	/**
	 * Browser which will be used for testing.
	 */
	final public static String BROWSER = config.getString("browser");

	/**
	 * URL of the application under test.
	 */
	final public static String URL_OF_AUT = config.getString("url.of.aut");

	/**
	 * Timeout in milliseconds after which a test step stops retrying doing a
	 * action.
	 */
	final public static String TIMEOUT = config.getString("timeout");

	/**
	 * Speed in milliseconds. What means that between each Selenium command
	 * Selenium waits x milliseconds where x is the speed.
	 */
	final public static String SPEED = config.getString("speed");

	/**
	 * Configures if the application under test shall be closed after test
	 * execution.
	 */
	final public static boolean CLOSE_TESTAPP_AFTER_EXECUTION = config
			.getBoolean("close.testapp.after.execution");

	/**
	 * Configures the browser log level.
	 */
	final public static String BROWSER_LOG_LEVEL = config
			.getString("browser.log.level");

	final public static boolean HIGHLIGHT_COMMANDS = config
			.getBoolean("highlight.elements");

	/**
	 * If execution of a action fails the program has to pause until it retries
	 * to execute this action again. This constant specifies how long the
	 * program will pause in milliseconds.
	 */
	final public static String PAUSE_BETWEEN_RETRIES = config
			.getString("pause.between.retries");

	/**
	 * This constant specifies the default prefix of an id locator. With the
	 * help of this prefix Selenium can be influenced how it locates elements.
	 * Selenium provides several mechanisms for that like regular expressions
	 * etc.
	 */
	final public static String IDENTIFICATION_PREFIX = config
			.getString("locator.prefix");

	/**
	 * It is nearly the same as {@link #IDENTIFICATION_PREFIX} with the
	 * difference that it is a suffix.
	 */
	final public static String IDENTIFICATION_SUFFIX = config
			.getString("locator.suffix");

	final public static String SCREENSHOT_ATTACHMENT_EXTENSION = config
			.getString("screenshot.attachment.extension");

	final public static String PAGE_SOURCE_ATTACHMENT_EXTENSION = config
			.getString("page.source.attachment.extension");

	/**
	 * This value is the maximum allowed difference of an current value and an
	 * expected value.
	 */
	final public static double VERIFICATION_NUMERIC_DIFFERENCE = config
			.getDouble("verification.numeric.difference", 0);

	/**
	 * The framework will stop the execution when this setting is set to true
	 * and in case of an error based by a verification command.
	 */
	final public static boolean BREAK_ON_VERIFICATION = config
			.getBoolean("break.on.verification.error");

	/**
	 * The framework will stop the execution when this setting is set to true
	 * and in case of an error based by a interaction command.
	 */
	final public static boolean BREAK_ON_INTERACTION = config
			.getBoolean("break.on.interaction.error");

}
